;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x21217439b78234068fc6eda896a9604d . "./code.cg2js/cg-acache.fasl")
(#x75a75cd125101e6a21673d8a68f470df . "./code.cg2js/cg-base.fasl")
(#x1a223bc7f3f67c39f3b3fba74f230a9a . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xa90c80299586703e9b071dd3512511d8 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x21631fbc527db4ed44ecea8d33d8768b . "./code.cg2js/cg-bitmap-stream.fasl")
(#x631109f1646e2aa4a2fc8a6a71d937f2 . "./code.cg2js/cg-button.fasl")
(#x792e916bc982a5c420415778a64e2968 . "./code.cg2js/cg-calendar.fasl")
(#x30ee79c4615a9af48ce8822333ad58db . "./code.cg2js/cg-caret.fasl")
(#xda0e4dc8f66695695eb1b31f475af25c . "./code.cg2js/cg-chart-or-plot.fasl")
(#xe4dcb8c9298bf2fd08b9d540894374dc . "./code.cg2js/cg-chart-widget.fasl")
(#xcdb12631b3523373dc2395be309a1d6d . "./code.cg2js/cg-check-box.fasl")
(#x6a74920526fb154a12359eeb0d33b023 . "./code.cg2js/cg-choice-list.fasl")
(#xa9e77464a2176c2ecbd5e4e3527fca83 . "./code.cg2js/cg-choose-printer.fasl")
(#x9384360c2bc9bcedc56f089603f44cc4 . "./code.cg2js/cg-class-grid.fasl")
(#x33d373cc914691dbe9e0d269f6036c10 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x30b9d3461a16949a5443a9c101aeadba . "./code.cg2js/cg-class-support.fasl")
(#xd7371c508975268171c17d5e72721e3c . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x24e037c0fbab65b8f954be3d5bdaf507 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x1b0dcbda9675ccc7f01c44a29b860b9e . "./code.cg2js/cg-clipboard.fasl")
(#x142aad1d19be3fc2267ad5c1f26c0ac0 . "./code.cg2js/cg-color-dialog.fasl")
(#x660c0d4c168349c2d746b8dd8636951e . "./code.cg2js/cg-combo-box.fasl")
(#x0123410407e163f4b6de1ef48f790633 . "./code.cg2js/cg-common-control.fasl")
(#xf9255f834aa37fc865074cdfa54bf54c . "./code.cg2js/cg-comtab.fasl")
(#xb40e8388e198c1bc99aa8de8f3bf17c3 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xa491bba6504311f794d580c7b447bdb9 . "./code.cg2js/cg-curve.fasl")
(#x3393592163f08603a4201b7de5b7f6c5 . "./code.cg2js/cg-dde-utils.fasl")
(#xf79d7bf809ea9f05a0cf9157bc8a7a63 . "./code.cg2js/cg-dev.fasl")
(#x08a5b7cb8f6422563f31baf60d70df5a . "./code.cg2js/cg-dialog-item.fasl")
(#xa7caef398d6bc62130d1fcccd67e3fbd . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x52df4f343a0c61ce80b005265ea54626 . "./code.cg2js/cg-directory-dialog.fasl")
(#x3118bc253fd0da77f920e5008eff8a42 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x70d6e70080a1c82e5a2320efb2ed46ae . "./code.cg2js/cg-drag-and-drop.fasl")
(#xf8548092f99e664eece1445951028476 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf4b5767a3b39cd5c6c5ae009166777af . "./code.cg2js/cg-drawable.fasl")
(#xd1bca468810f3855b8fc1804a449f5bb . "./code.cg2js/cg-dropping-outline.fasl")
(#xc7138fed312e00d6cefbf89f54561d6f . "./code.cg2js/cg-edit-in-place.fasl")
(#x276fd9f02a90806e6d037750c8d80250 . "./code.cg2js/cg-editable-text.fasl")
(#x9ebe1d298ebdb21ddc9bb4e0a3cd4d9a . "./code.cg2js/cg-file-dialog.fasl")
(#x77bc210d91fdbb63265392ec75288516 . "./code.cg2js/cg-fill-texture.fasl")
(#xabec8ab6c51f1c64b03b4ca0becc4693 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x9a67d15b5428a95a2edbd1df28884420 . "./code.cg2js/cg-font-dialog.fasl")
(#xfbc824ee2745a0f14249393f0c07379c . "./code.cg2js/cg-gesture-emulation.fasl")
(#x8fc1bb99dd72eb13152d32fec24bb048 . "./code.cg2js/cg-get-pixmap.fasl")
(#x3956d3ca52eb58571d9d9b71c8a83b9c . "./code.cg2js/cg-get-position.fasl")
(#x440797a59c73ab72a5714ed0c5d42c22 . "./code.cg2js/cg-graphics-context.fasl")
(#xf9dd3cc4cc1116688b157e7ad39c655e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xdab4117886319899041967902a762a06 . "./code.cg2js/cg-grid-widget.fasl")
(#xbe3af704199f97dae9a5410029360295 . "./code.cg2js/cg-group-box.fasl")
(#x6204ffdd85e6e927596a49899a20771d . "./code.cg2js/cg-header-control.fasl")
(#x83cc5986205b2b4907f0e2d2f427a43d . "./code.cg2js/cg-hotspot.fasl")
(#x1e7904a163e9c9e23aec8683005e4f20 . "./code.cg2js/cg-html-dialog.fasl")
(#xfa22b14e6fb45db879c122b89eb6dc3c . "./code.cg2js/cg-html-widget.fasl")
(#x2aed335e0288b7af14234111e62bc527 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x20f19df6ba3151a1d4fdf0b05ae9bd6b . "./code.cg2js/cg-icon.fasl")
(#x5e653fd5054e1f8a992321963b0b96f4 . "./code.cg2js/cg-ie.fasl")
(#x58c81677cd67424615e5fc0e3b48ad68 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xc3d49138b4eee0c86f197e46f5dee2bb . "./code.cg2js/cg-intersect.fasl")
(#x9f78f76a469df9d5702080c3f85d3472 . "./code.cg2js/cg-item-list.fasl")
(#x5a84e5c32d6f3e1c35dfa7652149a7bb . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x00cc17a70340f3a14d4398223a6aaf21 . "./code.cg2js/cg-lamp.fasl")
(#xf915834ee071b91154507af3a397fa43 . "./code.cg2js/cg-layout.fasl")
(#x79428747f7fa9cb9381dabc68da0a215 . "./code.cg2js/cg-lettered-menu.fasl")
(#x8d41479faf10b2fa3c9eefd58524ae29 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x7f5eaa4ab1b7fb45d9891f9553207e65 . "./code.cg2js/cg-lisp-text.fasl")
(#xa81617c1410387452e08a46ac176a77e . "./code.cg2js/cg-lisp-widget.fasl")
(#x8777a08c5a7bbf339f873fdaa9d3267d . "./code.cg2js/cg-list-view.fasl")
(#xe92cb9ad82baa97b48508e85ba17eed4 . "./code.cg2js/cg-mci.fasl")
(#x56bd56432b5509de8abf330de87f9bb4 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x04b73e7a5e92c6fc742105942645de7f . "./code.cg2js/cg-menu.fasl")
(#x2bf39b2babe9ea2db8a9c236d7768c63 . "./code.cg2js/cg-message-dialog.fasl")
(#x0baaaafd02de60a55879ab338f415cc5 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xa9ab7c7be823de320f03844bb37efd6a . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xe7d135e8d0d541979f7a4e20adb256ce . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x7478a80b9a84cca0f0677101dbe4b09e . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x58da6a99c31311f0fd3db6b2daaf0126 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x7d22d0f212b6b5045f8e64f5f7b7136d . "./code.cg2js/cg-nodes.fasl")
(#xc24fcb135216ee616c3c6dd6f3a60f79 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x849f08555d381826f19fcab01b0186dd . "./code.cg2js/cg-object-editor.fasl")
(#x7abcc30c66b6e1f38ff1aba44c9271ef . "./code.cg2js/cg-ocx.fasl")
(#xc2854346df97fbc020fb53e792c7b29b . "./code.cg2js/cg-os-widget.fasl")
(#x6d9e67a49f363e3b0b15e937e571e554 . "./code.cg2js/cg-os-window.fasl")
(#x13a69d25533ee47127ab3fd197fce50c . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xeed7394a0b5af890112d1f1486c598d3 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x80c408925573364c73bc11a30568c776 . "./code.cg2js/cg-outline.fasl")
(#x06cc4a06ed6e9ef5789dd99fe85711f1 . "./code.cg2js/cg-palette.fasl")
(#xd6b0ea9f96022fd27521367ee796bef8 . "./code.cg2js/cg-paren-matching.fasl")
(#x86f772407e2830576639f841599a9179 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xcfbd19da214a6f90f4ce34d67a6bc51c . "./code.cg2js/cg-picture-widget.fasl")
(#x6bcd15e16bb67f5690f85c3341625a9b . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xcc08647f6b7548ee8a732326f3871e1c . "./code.cg2js/cg-pixmap-printing.fasl")
(#x16070f9db20ddee0575d830dbc28ee16 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x508cebaba1f887f7c77eaf42908a5843 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x79185ad7f1f2d66822c9b19bc759eaab . "./code.cg2js/cg-pixmap.fasl")
(#x6c608128629dbad24dfe98012df5310b . "./code.cg2js/cg-player.fasl")
(#xe1a1353ed97ab0e99984107f15fd0a0b . "./code.cg2js/cg-plot-widget.fasl")
(#x06c44722ac5e98dc6f0868692b2e72a9 . "./code.cg2js/cg-printing.fasl")
(#x9043e171b772d40b9f42be241fc9b3b2 . "./code.cg2js/cg-progress-indicator.fasl")
(#x744b7ffa81387ec0d7c72cc77ac81dad . "./code.cg2js/cg-project-window.fasl")
(#x56bb2485e0eeba701c9da839714e390e . "./code.cg2js/cg-property.fasl")
(#x7e65f5bd8e7b27ea2ce9d43b9206fec1 . "./code.cg2js/cg-radio-button.fasl")
(#xa73ad6802bc175272aea222cbb16c337 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xf9e62db6331b7a47096c3aefb5e607ce . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x8a365d353fcfc36d7ab0e5d1e585856a . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x1ff56ac94bd01a9d8d29efdb028b51d8 . "./code.cg2js/cg-rich-edit.fasl")
(#x1b1a563bee5a405057f83b3187b13f96 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xcc1e91eabe368790a3b0bb5336b2eef1 . "./code.cg2js/cg-scaling-stream.fasl")
(#x2bb72b723c538f1bfd7e2ebcd06f59ca . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xa72e0384138b6b1e786a3c95725201e7 . "./code.cg2js/cg-scroll-bar.fasl")
(#x6143a6379ab817e10ed2c0db3525df1c . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x6593544caaec3e6bd645b58c96efba93 . "./code.cg2js/cg-selected-object.fasl")
(#x60701ead7ad24cfb72fbf94daa94d0cc . "./code.cg2js/cg-shortcut-menu.fasl")
(#xc2c5b43f052e4d29c4be0867265e0f49 . "./code.cg2js/cg-split-bar.fasl")
(#xfa9d008ec5824229a00430bae8adf3f2 . "./code.cg2js/cg-static-text.fasl")
(#xad08685d465e5ea82d1c20c923a09406 . "./code.cg2js/cg-status-bar.fasl")
(#xbd1cf01b123b4c545339136ef85a1986 . "./code.cg2js/cg-string-dialog.fasl")
(#x59e52d009fc33169cdf53998c815790e . "./code.cg2js/cg-tab-control.fasl")
(#xd9d88895d4fd837f1f09c3821160d2c1 . "./code.cg2js/cg-template-string.fasl")
(#x249fc1ea836998b02a6ba1260a766989 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x12f0631b195c85825ed5ec1f84db2629 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xade4e5b613729a4c68ca717760630fa8 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x9661b125100eb28218c94022e553e525 . "./code.cg2js/cg-text-or-combo.fasl")
(#x17f87718a6caf4d936395477aff7d0bd . "./code.cg2js/cg-text-widget.fasl")
(#x5abea8100e8b3943c7c37882374b5d65 . "./code.cg2js/cg-timer.fasl")
(#x50e4d2980a3e3b717f4403f806925ca9 . "./code.cg2js/cg-toggling-widget.fasl")
(#x21414558e14744f525f90fe3c970bb73 . "./code.cg2js/cg-toolbar.fasl")
(#xde5e7bd66376be271ad63f45ddc80622 . "./code.cg2js/cg-tooltip.fasl")
(#x36a5156017b8b625fe57a1fb0533ee9a . "./code.cg2js/cg-touch.fasl")
(#x8de6e55327a7bc9f791fafcd3d31864e . "./code.cg2js/cg-trackbar.fasl")
(#xcdcb72ef6dfd201ee8d9a86ddea5944b . "./code.cg2js/cg-tray.fasl")
(#x61aae2b9f50ed08c9970882ff4af59dd . "./code.cg2js/cg-up-down-control.fasl")
(#x216d6fc3bae60626c28b5f5945ebe0d0 . "./code.cg2js/cg-utility-dialog.fasl")
(#xe87a3a872ff7d01b5664bad268483b12 . "./code.cg2js/cg-web-browser-dde.fasl")
(#xd42a87842935c8c3367ada02c108c277 . "./code.cg2js/cg-web-browser.fasl")
(#x2ebfb97bb53ccdf7ba660c4416e32660 . "./code.cg2js/cg-wrap-string.fasl")
(#x51eb2b0413478a5bf5ea7b7d306cd950 . "./code.cg2js/cg-yes-no-list.fasl")
(#xa15627dda1acd3d9e79e9fe8c403d4a0 . "./code.cg2js/cg.fasl")
(#x425dd5ba05d6439727acb0d8a25f2eff . "./code.cg2js/cg-yes-no-string.fasl")
(#x4d9f99bad4211e8f610e783dbf6e3cc7 . "./code.idejs/ide-acache.fasl")
(#x06181f74acc78a5d7604e83e09d6f123 . "./code.idejs/ide-apropos.fasl")
(#x5dcc794902d35d97eb5ddd30e7582147 . "./code.idejs/ide-base.fasl")
(#x1cd33727c4e4a257561062587869a6f8 . "./code.idejs/ide-class-browser.fasl")
(#x97219cac2c412dab90826295d92ea0e3 . "./code.idejs/ide-class-grapher.fasl")
(#xa610b47db2fe51069b065e56591859d0 . "./code.idejs/ide-clipboard.fasl")
(#xbf9f0baaf0985fecf856ba9d9ed8a7bd . "./code.idejs/ide-dev.fasl")
(#xc9de36347ea826771ea72a27f3c353fa . "./code.idejs/ide-editor.fasl")
(#xa70c645a2aeedcbb0b397e937423edf7 . "./code.idejs/ide-find-definitions.fasl")
(#x423effde4a55225689e59e6a7d280d14 . "./code.idejs/ide-find-in-files.fasl")
(#x1b52ed4cc08d83aa55006dc89f6a805f . "./code.idejs/ide-form.fasl")
(#xc05e5686a0f7da9db648511a6ed889c6 . "./code.idejs/ide-grapher.fasl")
(#x4ee8dc2a01427882539faa3853d01fa6 . "./code.idejs/ide-inspector.fasl")
(#xaa2917cdc5b6a6d6fb9d83880c713218 . "./code.idejs/ide-intro.fasl")
(#xa12a2c195bdff24d25dd804661ce6360 . "./code.idejs/ide-keyshort.fasl")
(#x7e2d44134de3d25371711626c3a2760f . "./code.idejs/ide-navigator.fasl")
(#x99aaa0f1373f72950860ca68b3230e33 . "./code.idejs/ide-package-list.fasl")
(#xcce55705de0398bfcbc4474c6b35d761 . "./code.idejs/ide-patch-dialog.fasl")
(#x9bab519857f473b993d01ed720bf9fae . "./code.idejs/ide-process-browser.fasl")
(#xf1a589246591243c5776c4d4191614e6 . "./code.idejs/ide-project.fasl")
(#xf212119e3c14724557b72edb8f5973b3 . "./code.idejs/ide-trace.fasl")
(#x1e2c7f8670264a40a76853ff309c2903 . "./code.idejs/ide-window-list.fasl")
(#x48d0c564fc1bc2791be082167885bd91 . "./code.idejs/ide.fasl")
(#xf2773f648685397c1e60fb942db24e35 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
